@@ -1,19 +1,21 @@
-use crate::{
-    cache, docker, failure,
-    failure::Failure,
-    tar,
-    toastfile::{command, location, user, Task, Toastfile},
-};
-use std::{
-    collections::{HashMap, HashSet},
-    io::{Seek, SeekFrom},
-    path::PathBuf,
-    sync::{
-        atomic::{AtomicBool, Ordering},
-        Arc, Mutex,
+use {
+    crate::{
+        cache, docker, failure,
+        failure::Failure,
+        tar,
+        toastfile::{command, location, user, Task, Toastfile},
     },
+    std::{
+        collections::{HashMap, HashSet},
+        io::{Seek, SeekFrom},
+        path::PathBuf,
+        sync::{
+            atomic::{AtomicBool, Ordering},
+            Arc, Mutex,
+        },
+    },
+    tempfile::tempfile,
 };
-use tempfile::tempfile;
 
 // A context is an image that may need to be cleaned up.
 #[derive(Clone)]
@@ -54,6 +56,10 @@ pub fn run(
     // All relative paths are relative to where the toastfile lives.
     let mut toastfile_dir = PathBuf::from(&settings.toastfile_path);
     toastfile_dir.pop();
+    if toastfile_dir.components().peekable().peek().is_none() {
+        // [ref:source_dir_nonempty]
+        toastfile_dir.push(".");
+    }
 
     // Apply defaults.
     let location = location(toastfile, task);
